@@ -1,15 +1,16 @@
-# $Id$
+# $Id: Property.pm 57 2007-12-18 13:19:53Z asksol $
 # $Source$
-# $Author$
-# $HeadURL$
-# $Revision$
-# $Date$
+# $Author: asksol $
+# $HeadURL: https://class-dot.googlecode.com/svn/trunk/lib/Perl/Tags/ClassDot/Tag/Property.pm $
+# $Revision: 57 $
+# $Date: 2007-12-18 14:19:53 +0100 (Tue, 18 Dec 2007) $
 package Perl::Tags::ClassDot::Tag::Property;
 
 use strict;
 use warnings;
 use vars qw(@ISA);
-use version; our $VERSION = qv('2.0.0_04');
+use version; our $VERSION = qv('2.0.0_06');
+use 5.00600;
 
 @ISA = qw(Perl::Tags::Tag); ## no critic
 
