@@ -1,14 +1,10 @@
-# $Id$
+# $Id: Base.pm 57 2007-12-18 13:19:53Z asksol $
 # $Source$
-# $Author$
-# $HeadURL$
-# $Revision$
-# $Date$
+# $Author: asksol $
+# $HeadURL: https://class-dot.googlecode.com/svn/trunk/t/TestComplete/Base.pm $
+# $Revision: 57 $
+# $Date: 2007-12-18 14:19:53 +0100 (Tue, 18 Dec 2007) $
 package TestComplete::Base;
-
-use strict;
-use warnings;
-
 use Class::Dot qw(-new :std);
 
 property from_base => isa_String;
