@@ -1,16 +1,12 @@
-# $Id$
+# $Id: Type1.pm 57 2007-12-18 13:19:53Z asksol $
 # $Source$
-# $Author$
-# $HeadURL$
-# $Revision$
-# $Date$
+# $Author: asksol $
+# $HeadURL: https://class-dot.googlecode.com/svn/trunk/t/TestComplete/Type1.pm $
+# $Revision: 57 $
+# $Date: 2007-12-18 14:19:53 +0100 (Tue, 18 Dec 2007) $
 package TestComplete::Type1;
-use base 'TestComplete::Base';
-
-use strict;
-use warnings;
-
 use Class::Dot qw(-new :std);
+extends 'TestComplete::Base';
 
 property in_type1 => isa_String();
 
